#ifndef TOUCHPARSER_H_
#define TOUCHPARSER_H_

#include <stdint.h>
#include <Arduino.h>

enum TouchPressEvent {
	kTouchNone = 0x00,
	kTouchPress = 0x73,
	kTouchRelease = 0x72
};

struct TouchEvent {
	TouchPressEvent event;
	uint16_t x;
	uint16_t y;
	
	TouchEvent()
	  : event(kTouchNone)
	  , x(0)
	  , y(0)
        {}
};

class TouchParser {
public:
	TouchParser(Stream& stream)
	  : stream(stream)
	  , buffer_index(0)
	  , buffer()
	  , last_recv_ms(0)
	{}
	
	TouchEvent read();
		
private:
	Stream& stream;
	
	uint8_t buffer_index;
	uint8_t buffer[10 + 1];
	
	uint32_t last_recv_ms;
	
	bool receive();
	bool check_packet() const;
	void reset();
	bool extract_event(TouchEvent& event) const;
	bool extract_x(TouchEvent& event) const;
	bool extract_y(TouchEvent& event) const;
	uint16_t extract_coord(uint8_t offset) const;
};


inline
TouchEvent TouchParser::read() {
	if(!receive()) {
		return TouchEvent();
	}
	
	TouchEvent event;
	if(!check_packet()
		|| !extract_event(event)
		|| !extract_x(event)
		|| !extract_y(event))
	{
		event = TouchEvent();
	}
	
	reset();
	return event;
}

inline
bool TouchParser::receive() {
	const uint32_t kTimeout = 1000;
	
	if(stream.available()) {
		for(uint8_t i = 0; i < 50; ++i) {
			int c = stream.read();
			//Serial.println(c,HEX); //print byte received
			if(c < 0) { break; }
			if(buffer_index == 0) {
				if(c == 0xAA) {
					buffer[buffer_index++] = c;
				} else {
					// skip
				}
			} else if(buffer_index == 10 + 1) {
				// skip
			} else {
				buffer[buffer_index++] = c;
				if(buffer_index >= 5) {
					if(buffer[buffer_index-4+0] == 0xCC
						&& buffer[buffer_index-4+1] == 0x33
						&& buffer[buffer_index-4+2] == 0xC3
						&& buffer[buffer_index-4+3] == 0x3C)
					{
						last_recv_ms = millis();
						return true;
					}
				}
			}
		}
		last_recv_ms = millis();
	}
	else {
		if(buffer_index && (millis() - last_recv_ms >= kTimeout)) {
			Serial.write('#');Serial.println(buffer_index);
			reset();
			return false;
		}
	}
	
	return false;
}

inline
void TouchParser::reset() {
	buffer_index = 0;
}

inline
bool TouchParser::check_packet() const {
	return buffer_index == 1+5+4
		&& buffer[0] == 0xAA
		&& buffer[1+5+0] == 0xCC
		&& buffer[1+5+1] == 0x33
		&& buffer[1+5+2] == 0xC3
		&& buffer[1+5+3] == 0x3C
		;
}

inline
bool TouchParser::extract_event(TouchEvent& event) const {
	uint8_t c = buffer[1 + 0];
	
	if(c == kTouchPress || c == kTouchRelease) {
		event.event = TouchPressEvent(c);
		return true;
	} else {
		return false;
	}
}

inline
bool TouchParser::extract_x(TouchEvent& event) const {
	event.x = extract_coord(1+1);
	
	return true;
}

inline
bool TouchParser::extract_y(TouchEvent& event) const {
	event.y = extract_coord(1+3);
	return true;
}

inline
uint16_t TouchParser::extract_coord(uint8_t offset) const {
	return (buffer[offset] << 8) | buffer[offset + 1];
}


#endif //TOUCHPARSER_H_
